
/**
 * UPC is the identifier for a Product.
 * 
 * @author  J. Dalbey
 * @version 1
 */
public class UPC
{
    public final String upc;

    /**
     * Constructor for objects of class UPC
     */
    public UPC(String upc)
    {
        this.upc = upc;
    }
    public String toString()
    {
        return upc;
    }
    /** Verify the upc is valid by applying the check digit algorithm */
    public boolean verify()
    {
        int checkDigit;
        if (upc == null) return false;
        if (upc.length() != 12) return false;
        //calc sum of odd-numbered positions & multiply by 3
        int oddNumberedPos = (Character.getNumericValue(upc.charAt(0)) + 
                                Character.getNumericValue(upc.charAt(2)) + 
                                Character.getNumericValue(upc.charAt(4)) +
                                Character.getNumericValue(upc.charAt(6)) +
                                Character.getNumericValue(upc.charAt(8)) +
                                Character.getNumericValue(upc.charAt(10))) * 3;
        int evenNumberedSum = Character.getNumericValue(upc.charAt(1)) + 
                                Character.getNumericValue(upc.charAt(3)) + 
                                Character.getNumericValue(upc.charAt(5)) +
                                Character.getNumericValue(upc.charAt(7)) +
                                Character.getNumericValue(upc.charAt(9));
                                
        Integer result = new Integer(oddNumberedPos + evenNumberedSum);
        String strResult = result.toString();
        int lastDigit = 
            Character.getNumericValue(strResult.charAt(strResult.length() - 1));
        
        //if lastDigit == 0
        if (lastDigit == 0)
            checkDigit = 0;
        else
            checkDigit = 10 - lastDigit;
            
        return checkDigit == Character.getNumericValue(upc.charAt(11));
        
    }
    /** This method is needed because UPC is a key in a HashMap */
    @Override
    public int hashCode() 
    {
        return this.toString().hashCode();
    }
 
    /** This method is needed because UPC is a key in a HashMap */
    @Override
    public boolean equals(Object obj) 
    {
        // Check the object is valid and is of the correct class type
        if ( obj == null ) return false;
        if ( this.getClass() != obj.getClass() ) return false;
         
        // It is now safe to cast and compare the classes data members
        String name = ((UPC)obj).toString();
        return this.toString().equals(name);
    }
}
